"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = fetchProps;

var _regenerator = _interopRequireDefault(require("@babel/runtime/regenerator"));

var _asyncToGenerator2 = _interopRequireDefault(require("@babel/runtime/helpers/asyncToGenerator"));

var _isomorphicUnfetch = _interopRequireDefault(require("isomorphic-unfetch"));

var _serviceWorker = require("../serviceWorker");

/**
 * Creates a `getInitialProps` props function that fetches props from an API endpoint. Use this
 * in conjunction with `react-storefront/hooks/useLazyStore` to display a skeleton with partial
 * data while fetching the full data for the page from the server.
 *
 * The returned function will skip displaying the skeleton if the API response can be served from
 * the browser's cache.
 *
 * Example:
 *
 * ```js
 * import useLazyStore from 'react-storefront/hooks/useLazyStore'
 * import fetchProps from 'react-storefront/props/fetchProps'
 *
 * function Product(lazyProps) {
 *   const [store, updateStore] = useLazyStore(lazyProps)
 *   const { product } = store.pageData
 *
 *   // store.loading will be true while fetching product data from the server
 *
 *   // store.pageData will be populated with the `pageData` prop provided to the <Link> element
 *   // that was clicked.  In this way you can provide partial data to a page.  For example:
 *   //
 *   // <Link href="/p/[productId]" as={`/p/${product.id}`} pageData={{ product }}>{product.name}</Link>
 *
 *   return (
 *     <Grid container spacing={4}>
 *       <Grid item xs={12}>
 *         { product.name ? <Typography variant="h1">{product.name}</Typography> : <Skeleton style={{ height: 16 }}/> }
 *       </Grid>
 *       // render the rest of the PDP
 *     </Grid>
 *   )
 * }
 *
 * Product.getInitialProps = withCaching({
 *   browser: true
 *   edge: {
 *     maxAgeSeconds: 1000,
 *     key: createCustomCacheKey().addCookie('currency')
 *   }
 * })(fetchProps(({ query }) => `/api/p/${encodeURIComponent(query.productId)}`))
 * ```
 *
 * @param {Function} createAPIURL
 */
function fetchProps(createAPIURL) {
  return function (options) {
    var server = typeof window === 'undefined';
    var host = server ? options.req.headers['host'] : '';
    var protocol = server ? host.startsWith('localhost') ? 'http://' : 'https://' : '';
    var apiURL = "".concat(protocol).concat(host).concat(createAPIURL(options));
    return createLazyProps(options.asPath, apiURL, options.rsf_app_shell === '1');
  };
}

var serviceWorkerReady = false;
var requestId = 0;

function nextRequestId() {
  return requestId++;
}

if (typeof window !== 'undefined') {
  (0, _serviceWorker.waitForServiceWorker)().then(function () {
    serviceWorkerReady = true;
  });
}

function createLazyProps(_x, _x2, _x3) {
  return _createLazyProps.apply(this, arguments);
}

function _createLazyProps() {
  _createLazyProps = (0, _asyncToGenerator2["default"])(
  /*#__PURE__*/
  _regenerator["default"].mark(function _callee(as, apiURL, shell) {
    var doFetch, rsf, res;
    return _regenerator["default"].wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            if (typeof window === 'undefined') {
              if (apiURL.indexOf('?') === -1) {
                apiURL = apiURL + '?_includeAppData=1';
              } else {
                apiURL = apiURL + '&_includeAppData=1';
              }
            }

            doFetch = function doFetch() {
              var onlyHit = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : false;

              if (onlyHit && process.env.NODE_ENV === 'development') {
                return Promise.resolve({
                  status: 204
                });
              }

              var headers = {
                'x-rsf-api-version': process.env.RSF_API_VERSION
              };

              if (onlyHit) {
                headers['x-rsf-client-if'] = 'cache-hit';
              }

              return (0, _isomorphicUnfetch["default"])(apiURL, {
                cache: 'force-cache',
                headers: headers
              });
            };

            if (!(typeof window === 'undefined')) {
              _context.next = 12;
              break;
            }

            if (!shell) {
              _context.next = 7;
              break;
            }

            return _context.abrupt("return", {
              lazy: apiURL
            });

          case 7:
            _context.next = 9;
            return doFetch();

          case 9:
            return _context.abrupt("return", _context.sent.json());

          case 10:
            _context.next = 24;
            break;

          case 12:
            // client
            rsf = window.history.state.rsf;

            if (!(rsf && rsf[as])) {
              _context.next = 17;
              break;
            }

            return _context.abrupt("return", {
              pageData: rsf[as],
              requestId: nextRequestId()
              /* Need to send requestId or going back twice won't update the state correctly */

            });

          case 17:
            if (!serviceWorkerReady) {
              _context.next = 23;
              break;
            }

            _context.next = 20;
            return doFetch(true);

          case 20:
            res = _context.sent;

            if (!(res.status !== 204)) {
              _context.next = 23;
              break;
            }

            return _context.abrupt("return", res.json());

          case 23:
            return _context.abrupt("return", {
              lazy: apiURL,
              // requestId forces useLazyProps to refetch from the server, which should be done every time
              // getInitialProps is called, otherwise pages that use replace state to add data to the query string
              // when state changes such as subcategory won't properly reset when the user clicks a link back
              // to the same page without the query string
              requestId: nextRequestId()
            });

          case 24:
          case "end":
            return _context.stop();
        }
      }
    }, _callee);
  }));
  return _createLazyProps.apply(this, arguments);
}
//# sourceMappingURL=fetchProps.js.map